

#import python modules
import numpy as np
import time

#import compiled fortran library
import ex6lib

import sys
import os
import atomwrite

#whether or not to use the 3d visualization
UseVisual = True
if UseVisual:
    #import custom visualization library
    import atomvis


#========== GLOBAL VARIABLES ==========
#distance cutoff for pairwise interactions
Cut = 3.0
#set the random number seed; useful for debugging
np.random.seed(342324)


#NOTE:
#everything below assumes unit atomic masses,
#such that forces = accelerations.


def CalcEnergy(Pos):
    """Wrapper for energy calculation.
Input:
    Pos: (N,3) position array
Output:
    PEnergy: potential energy, float
"""
    global M, L, Cut
    return ex6lib.calcenergy(Pos, M, L, Cut)


def CalcEnergyForces(Pos, Forces):
    """Wrapper for energy and force calculation.
Input:
    Pos: (N,3) position array
    Forces: (N,3) force array
Output:
    PEnergy: potential energy, float
    Forces: (N,3) force array (updated)
"""
    global M, L, Cut
    return ex6lib.calcenergyforces(Pos, M, L, Cut, Forces)


def InitPositions(N, L):
    """Returns an array of initial positions of each atom,
placed on a cubic lattice for convenience.
Input:
    N: number of atoms
    L: box length
Output:
    Pos: (N,3) array of positions
"""
    #make the position array
    Pos = np.zeros((N,3), float)
    #compute integer grid # of locations for cubic lattice
    NLat = int(N**(1./3.) + 1.)
    LatSpac = L / NLat
    #make an array of lattice sites
    r = LatSpac * np.arange(NLat, dtype=float) - 0.5*L
    #loop through x, y, z positions in lattice until done
    #for every atom in the system
    i = 0
    for x in r:
        for y in r:
            for z in r:
                Pos[i] = np.array([x,y,z], float)
                #add a random offset to help initial minimization
                Offset = 0.1 * LatSpac * (np.random.rand(3) - 0.5)
                Pos[i] = Pos[i] + Offset
                i += 1
                #if done placing atoms, return
                if i >= N:
                    return Pos
    return Pos


def RescaleVelocities(Vel, T):
    """Rescales velocities in the system to the target temperature.
Input:
    Vel: (N,3) array of atomic velocities
    T: target temperature
Output:
    Vel: same as above
"""
    #recenter to zero net momentum (assuming all masses same)
    Vel = Vel - Vel.mean(axis=0)
    #find the total kinetic energy
    KE = 0.5 * np.sum(Vel * Vel)
    #find velocity scale factor from ratios of kinetic energy
    VScale = np.sqrt(1.5 * len(Vel) * T / KE)
    Vel = Vel * VScale
    return Vel  


def InitVelocities(N, T):
    """Returns an initial random velocity set.
Input:
    N: number of atoms
    T: target temperature
Output:
    Vel: (N,3) array of atomic velocities
"""
    Vel = np.random.rand(N, 3)
    Vel = RescaleVelocities(Vel, T)
    return Vel


def InitAccel(Pos):
    """Returns the initial acceleration array.
Input:
    Pos: (N,3) array of atomic positions
Output:
    Accel: (N,3) array of acceleration vectors
"""
    #get the acceleration from the forces
    PEnergy, Accel = CalcEnergyForces(Pos, np.zeros_like(Pos))
    return Accel


def InstTemp(Vel):
    """Returns the instantaneous temperature.
Input:
    Vel: (N,3) array of atomic velocities
Output:
    Tinst: float
"""
    return np.sum(Vel * Vel) / (3. * len(Vel))

M = 6.
N = 210.
Nt = (N/M)
dc = 2.5
NAtom = 210.
rho = 0.25
L = (N / rho)**(1./3.)
Cut = 3.0
rc = 3.0
Temp = 1.5
dt = 0.001
EFracTolLS = 1.e-8
EFracTolCG = 1.e-10
dx = 0.001
agg = 0
num = 0
Dim2 = 1
Dim = 3
RescaleFreq = 100
PdbFreq = 60

DisplayFreq = .1

Names = ['C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H',
         'C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H',
         'C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H',
         'C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H',
         'C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H','C','H','H','H','H','H']
Pdb = atomwrite.pdbfile("movie11.pdb", L, AtomNames = Names)

Pos = InitPositions(N, L)
Vel = InitVelocities(N, Temp)
Accel = InitAccel(Pos)
TotE =[]
PE = []


PEnergy = CalcEnergy(Pos)
print PEnergy
MaxSteps = 300000
i = 0
while i < MaxSteps:
    Pos, Vel, Accel, KEnergy, PEnergy = ex6lib.vvintegrate(Pos, Vel, Accel, M, L, Cut, dt)
    if UseVisual:
        if atomvis.Initialized:
            #update the positions
            atomvis.Update(Pos)
        else:
            #initialize the visualization window
            atomvis.Init(Pos)
    if i%5000 == 0:
        print PEnergy

    if i % PdbFreq == 0 :
        Pdb.write(Pos)
    i +=1
    
    
    if i < 200000:
        if i % RescaleFreq == 0:
            Vel = RescaleVelocities(Vel, Temp)
    if i >= 200000:
        TotE.append(PEnergy + KEnergy)
        PE.append(PEnergy)

Count = np.ones((Nt,1), int)
Count = ex6lib.calcdistance(Pos,Count, M, L, dc, Dim, NAtom, Dim2, Nt)
agg = Count.max()
for x in Count:
    if x == agg:
        num += 1

Pdb.close()
TotE = np.array(TotE, float)
PE = np.array(PE, float)
standev = TotE.std()
average = TotE.mean()
avgP = PE.mean()
print standev, average, Temp, rho
print Count, agg, num
print avgP


    




